
; Definitionen fr Tiny45

; Belegungen der Atmel-Ports:

;*****************************************************************

;	PB0		MOSI			PCINT0			
;   PB1		MISO	OC1A	PCINT1	SDI RFM02	
;   PB2		SCK		INT0	PCINT2	SCK RFM02
;   PB3				/OC1B	PCINT3	CS  RFM12 [nSEL] 	 			
;   PB4				OC1B	PCINT4	IRQ RFM02 [nIRQ]

;	FSK: Pullup

;*****************************************************************

.equ	SENSOR			= 6

;--- Allgemeines -----------------------------------------------

.equ    F_CPU           = 8000000
.equ    CYCLES_PER_US   = ((F_CPU+500000)/1000000)

.equ	BATT_OK			= 0xA5
.equ	BATT_LOW		= 0x5A

.equ	TEMP_OFFSET		= 267						; Offset des Tiny-Sensors
	
;**************************** Port-Bits ***************************

.equ	RFM02_PORT		= PORTB
.equ	RFM02_PIN		= PINB
.equ	RFM02_DDR		= DDRB

.equ	RFM02_SDI		= PB1
.equ	RFM02_SCK		= PB2
.equ	RFM02_CS		= PB3
.equ	RFM02_IRQ		= PB4


;**************************** Ram-Reservierungen ******************

;--- Internes Ram ------------------------------------------------

.DSEG

.ORG    0x060										; 160 Byte allgemein

DATA_BUF:
SENSOR_NR:		.byte	 2
VORZEICHEN:		.byte    1
TEMPERATUR:		.byte	 4
FEUCHTE:		.byte	 3
LUFTDRUCK:		.byte	 6
BATTERIE:		.byte	 1
CHECKSUMME:		.byte	 1							; PAKET_LEN = 18
DATA_END:

.equ	PAKET_LEN		= DATA_END - DATA_BUF

;**************************** Register ****************************

;--- lower Register -----------------------------------------------

.def	SREG_SAVE	= r0
.def	NULL		= r1

.def	MUL_1_L		=r2				; multiplicand low byte
.def	MUL_1_H		=r3				; multiplicand high byte
.def	MUL_2_L		=r4				; multiplier low byte
.def	MUL_2_H		=r5				; multiplier high byte
.def	MUL_E_0		=r6				; result byte 0 (LSB)
.def	MUL_E_1		=r7				; result byte 1
.def	MUL_E_2		=r8				; result byte 2
.def	MUL_E_3		=r9				; result byte 3 (MSB)

.def	BCD0		= r13			; BCD Wert Digits 1 und 0
.def	BCD1		= r14			; BCD Wert Digits 3 und 2
.def	BCD2		= r15			; BCD Wert Digit 4

.equ	BCD0_ADR	= 13			; Address von tBCD0
.equ	BCD2_ADR	= 15			; Address von tBCD1

;--- higher Register ----------------------------------------------

.def	TEMP_L		= r16
.def	TEMP_H		= r17

.def	SPANNUNG_L	= r18
.def	SPANNUNG_H	= r19

.def	TEMP_3		= r22                   		; Temp-Register
.def	TEMP_2		= r23                   		; Temp-Register
.def	TEMP_1		= r24                   		; Temp-Register
.def	TEMP_0		= r25							; Temp-Register

; r26/r27, r28/r29, r30/r31 fr X,Y,Z reserviert 

;--- Macros -------------------------------------------------------

.macro  load_p                      ; ldt 16Bit @1 nach @0 High und @0 Low

        ldi     @0H,HIGH(@1)
        ldi     @0L,LOW(@1)

.endmacro

