;*********************** Allgemeine Subroutinen ****************************

;*************************** wartet ca. 20ms *******************************
; Sub: Wartet ca. 20ms
; Parameter:	-
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

wait_20ms:
		push	XH
		push	XL

		ldi		XH,high(CYCLES_PER_US*20/4)			; 4 Anzahl der Zyklen fr die Schleife
		ldi		XL,low(CYCLES_PER_US*20/4)

wait_loop:
		sbiw	X,1
		brne	wait_loop

		pop		XL
		pop		XH
		ret

;*********************************************************************
;  Berechnet Checksumme
;  In:  DATA_BUF
;  Out: DATA_BUF
;  Scratch-Reg: Z, TEMP_0
;*********************************************************************

checksum:
		load_p	Z,DATA_BUF
		ldi		TEMP_1,PAKET_LEN-1
		clr		TEMP_2

checksum_loop:
		ld		TEMP_0,Z+
		add		TEMP_2,TEMP_0
		dec		TEMP_1
		brne	checksum_loop

		st		Z,TEMP_2					; Prfsumme speichern

		ret

;*********************************************************************
;  Berechnet Checksumme
;  In:  DATA_BUF
;  Out: DATA_BUF
;  Scratch-Reg: Z, TEMP_0
;*********************************************************************

init_text:
		load_p	Z,(2*text)
		load_p	Y,DATA_BUF				; Beginn Text

		ldi		TEMP_1,PAKET_LEN

init_text_loop:
		lpm		TEMP_0,Z+
		st		Y+,TEMP_0
		dec		TEMP_1
		brne	init_text_loop

		ret

;*********************************************************************
;  Initialisiert DATA_BUF mit 0x55 -> kein Sensor
;  In:  DATA_BUF
;  Out: DATA_BUF
;  Scratch-Reg: Z, TEMP_0
;*********************************************************************

init_data_buf:
		ldi		TEMP_1,PAKET_LEN-2							; Sensornummer drinlassen
		ldi		TEMP_0,0x55
		load_p	Z,DATA_BUF+2

init_data_buf_loop:
		st		Z+,TEMP_0
		dec		TEMP_1
		brne	init_data_buf_loop

		ldi		TEMP_0, BATT_OK								; Batteriestatus erstmal ok setzen
		sts		BATTERIE,TEMP_0

		ret	

;************************** Sensor Feuchte *********************************
; Sub: Berechnet Temperatur des FOST02
; Parameter:	ADC-Wert in TEMP_H/TEMP_L
; Return:		Packed BCD in BCD2...BCD0 (0 - 65535)
; Scratch-Reg:	TEMP_0, TEMP_1
;***************************************************************************

convert_temp:
		ldi		TEMP_0,' '

		subi	TEMP_L,low(TEMP_OFFSET)
		sbci	TEMP_H,high(TEMP_OFFSET)
		brcc	convert_temp_pos

		clr		TEMP_0
		clr		TEMP_1
		sub		TEMP_0,TEMP_L
		sbc		TEMP_1,TEMP_H

		mov		TEMP_L,TEMP_0
		mov		TEMP_H,TEMP_1

		ldi		TEMP_0,'-'
		
convert_temp_pos:
		sts		VORZEICHEN,TEMP_0

		mov		TEMP_0,TEMP_L	;SPANNUNG_L
		mov		TEMP_1,TEMP_H	;SPANNUNG_H
		rcall	show_16bit_wert_temp

		ret

; 16x16 bit Multiplikation siehe ( AVR200 )  


;************************** Sensor Feuchte *********************************
; Sub: unsigned Multiplikation 16Bit * 16Bit -> 32Bit
; Parameter:	MUL_1_H, MUL_1_L / MUL_2_H, MUL_2_L
; Return:		MUL_E_3...MUL_E_0
; Scratch-Reg:	TEMP_0
;***************************************************************************

sub_mul_16:
		clr		MUL_E_3				;oberste 2 Ergebnis-Bytes loschen
		clr		MUL_E_2
		ldi		TEMP_0,16			;Bitzhler

sub_mul_16_loop:
		lsr		MUL_2_H
		ror		MUL_2_L
		brcc	sub_mul_16_no_add	;Faktor 2 Bit 0 = 0

		add		MUL_E_2,MUL_1_L		;addiert Faktor 1 Low zu Byte 2 des Ergebnis
		adc		MUL_E_3,MUL_1_H		;addiert Faktor 1 High zu Byte 2 des Ergebnis

sub_mul_16_no_add:
		ror		MUL_E_3				;Ergebnis Byte 3 schieben
		ror		MUL_E_2
		ror		MUL_E_1
		ror		MUL_E_0

		dec		TEMP_0				;nchstes Bit
		brne	sub_mul_16_loop	

		ret

;*********************************************************************
;  16Bit bin to BCD (eigentlich packed-BCD, immer 2 Stellen in einem Register)
;  In: TEMP_0 (L), TEMP_1 (H)
;  Out: BCD2 (x/4), BCD1 (3/2), BCD0 (1/0)
;  Scratch-Reg: TEMP_0, TEMP_1, TEMP_2, TEMP_3, ZH, ZL
;*********************************************************************

bin2BCD16:
		ldi		TEMP_2,16					; Bitzhler
		clr		BCD2						; Ergebnis lschen (3 bytes)
		clr		BCD1
		clr		BCD0
		clr		ZH

bin2BCD16_loop:
		lsl		TEMP_0						; Input schieben
		rol		TEMP_1						; durch alle Bytes
		rol		BCD0				
		rol		BCD1	
		rol		BCD2
		dec		TEMP_2						; Bitzhler
		breq	bin2BCD16_end				; fertig ?

		ldi		r30,BCD2_ADR+1				; Z auf Ergebnis MSB + 1

bin2BCD16_encode:
		ld		TEMP_3,-Z					; (Z) holen mit Pre-decrement
		subi	TEMP_3,-$03					; + $03
		sbrc	TEMP_3,3					; wenn Bit 3 nicht 0
		st		Z,TEMP_3					; speichern
		ld		TEMP_3,Z					; (Z) holen
		subi	TEMP_3,-$30					; + $30
		sbrc	TEMP_3,7					; wenn Bit 7 nicht 0
		st		Z,TEMP_3					; speichern
		cpi		ZL,BCD0_ADR					; alle erledigt??
		brne	bin2BCD16_encode			; nein, weiter

		rjmp	bin2BCD16_loop

bin2BCD16_end:
		ret

;*********************************************************************
;  Schreibt Temperaturwert als ASCII in DATA_BUF
;  In: TEMP_1 (H), TEMP_0 (L)
;  Out: BCD2 (x/4), BCD1 (3/2), BCD0 (1/0)
;  Scratch-Reg: Z, TEMP_0
;*********************************************************************

show_16bit_wert_temp:
		rcall	bin2BCD16

		load_p	Z,TEMPERATUR

		mov		TEMP_0,BCD0
		swap	TEMP_0
		andi	TEMP_0,$0F
		ori		TEMP_0,'0'
		st		Z+,TEMP_0

		mov		TEMP_0,BCD0
		andi	TEMP_0,$0F
		ori		TEMP_0,'0'
		st		Z+,TEMP_0

		ldi		TEMP_0,'0'
		st		Z+,TEMP_0

		ldi		TEMP_0,'0'
		st		Z+,TEMP_0

		ret



